#include <stdlib.h>
#include <string.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
#define YYPREFIX "yy"
#line 2 "parse.y"

/*
 * Copyright (c) 2021, 2022, 2023 Omar Polo <op@omarpolo.com>
 * Copyright (c) 2018 Florian Obser <florian@openbsd.org>
 * Copyright (c) 2004, 2005 Esben Norby <norby@openbsd.org>
 * Copyright (c) 2004 Ryan McBride <mcbride@openbsd.org>
 * Copyright (c) 2002, 2003, 2004 Henning Brauer <henning@openbsd.org>
 * Copyright (c) 2001 Markus Friedl.  All rights reserved.
 * Copyright (c) 2001 Daniel Hartmeier.  All rights reserved.
 * Copyright (c) 2001 Theo de Raadt.  All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "gmid.h"

#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>

#include "log.h"

struct conf *conf;

static const char	*default_host = NULL;
static uint16_t		 default_port = 1965;

TAILQ_HEAD(files, file)		 files = TAILQ_HEAD_INITIALIZER(files);
static struct file {
	TAILQ_ENTRY(file)	 entry;
	FILE			*stream;
	char			*name;
	size_t	 		 ungetpos;
	size_t			 ungetsize;
	u_char			*ungetbuf;
	int			 eof_reached;
	int			 lineno;
	int			 errors;
} *file, *topfile;

struct file	*pushfile(const char *, int);
int		 popfile(void);
int		 yyparse(void);
int		 yylex(void);
void		 yyerror(const char *, ...)
    __attribute__((__format__ (printf, 1, 2)))
    __attribute__((__nonnull__ (1)));
void		 yywarn(const char *, ...)
    __attribute__((__format__ (printf, 1, 2)))
    __attribute__((__nonnull__ (1)));
int		 kw_cmp(const void *, const void *);
int		 lookup(char *);
int		 igetc(void);
int		 lgetc(int);
void		 lungetc(int);
int		 findeol(void);

/*
 * #define YYDEBUG 1
 * int yydebug = 1;
 */

TAILQ_HEAD(symhead, sym) symhead = TAILQ_HEAD_INITIALIZER(symhead);
struct sym {
	TAILQ_ENTRY(sym)	 entry;
	int			 used;
	int			 persist;
	char			*name;
	char			*val;
};

int	 symset(const char *, const char *, int);
char	*symget(const char *);

char		*ensure_absolute_path(char*);
int		 check_block_code(int);
char		*check_block_fmt(char*);
int		 check_strip_no(int);
int		 check_port_num(int);
int		 check_prefork_num(int);
void		 advance_loc(void);
void		 advance_proxy(void);
void		 parsehp(char *, char **, const char **, const char *);
int		 fastcgi_conf(const char *, const char *);
void		 add_param(char *, char *);
int		 getservice(const char *);
void		 listen_on(const char *, const char *);

static struct vhost		*host;
static struct location		*loc;
static struct proxy		*proxy;
static char			*current_media;
static int			 errors;

typedef struct {
	union {
		char	*string;
		int	 number;
	} v;
	int lineno;
} YYSTYPE;

#define YYSTYPE YYSTYPE

#line 133 "y.tab.c"
#define ACCESS 257
#define ALIAS 258
#define AUTO 259
#define BLOCK 260
#define CA 261
#define CERT 262
#define CHROOT 263
#define CLIENT 264
#define COMBINED 265
#define COMMON 266
#define CONDENSED 267
#define DEFAULT 268
#define FACILITY 269
#define FASTCGI 270
#define FOR_HOST 271
#define INCLUDE 272
#define INDEX 273
#define IPV6 274
#define KEY 275
#define LANG 276
#define LEGACY 277
#define LISTEN 278
#define LOCATION 279
#define LOG 280
#define OCSP 281
#define OFF 282
#define ON 283
#define PARAM 284
#define PORT 285
#define PREFORK 286
#define PROTO 287
#define PROTOCOLS 288
#define PROXY 289
#define RELAY_TO 290
#define REQUIRE 291
#define RETURN 292
#define ROOT 293
#define SERVER 294
#define SNI 295
#define SOCKET 296
#define STRIP 297
#define STYLE 298
#define SYSLOG 299
#define TCP 300
#define TOEXT 301
#define TYPE 302
#define TYPES 303
#define USE_TLS 304
#define USER 305
#define VERIFYNAME 306
#define ERROR 307
#define STRING 308
#define NUM 309
#define YYERRCODE 256
const short yylhs[] =
	{                                        -1,
    0,    0,    0,    0,    0,    0,    0,    0,    6,    1,
    1,    3,    3,    4,    4,    7,    8,    8,    8,    8,
    8,    8,    8,   11,   11,   13,   13,   14,   14,   14,
   14,   14,   14,   14,   14,   15,    9,    9,   16,   16,
   16,   16,    5,    5,   17,   17,   17,   17,   17,   17,
   17,   17,   17,   22,   19,   21,   21,    2,    2,    2,
   24,   24,   23,   23,   25,   25,   25,   25,   25,   25,
   25,   25,   26,   18,   18,   27,   27,   20,   20,   20,
   20,   20,   20,   20,   20,   20,   20,   20,   20,   28,
   28,   28,   28,   29,   29,   30,   30,   30,   30,   30,
   30,   10,   31,   31,   35,   32,   32,   34,   34,   37,
   33,   12,   12,   12,   36,   36,
};
const short yylen[] =
	{                                         2,
    0,    3,    2,    3,    3,    3,    3,    3,    2,    1,
    1,    2,    1,    1,    1,    3,    2,    2,    1,    2,
    2,    2,    2,    5,    2,    0,    3,    2,    2,    2,
    2,    2,    3,    2,    1,    0,    7,    2,    0,    3,
    3,    3,    1,    1,    2,    2,    2,    2,    4,    3,
    5,    5,    1,    0,    7,    0,    2,    0,    2,    2,
    2,    3,    0,    3,    2,    2,    2,    3,    4,    2,
    2,    2,    0,    7,    2,    0,    3,    3,    4,    3,
    1,    3,    1,    2,    2,    2,    4,    2,    2,    5,
    2,    2,    2,    0,    3,    4,    2,    5,    3,    5,
    2,    5,    3,    2,    0,    4,    1,    2,    1,    1,
    2,    2,    2,    0,    1,    0,
};
const short yydefred[] =
	{                                      1,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    3,    0,    0,    0,    0,    0,   19,
    8,   38,   13,    0,    9,   11,   10,   18,    0,    0,
    0,    0,   25,   20,   21,    0,    0,    0,    0,    0,
    2,    4,    5,    6,    7,   12,    0,   30,   29,   31,
   32,    0,   34,    0,    0,   26,    0,    0,    0,    0,
  112,  113,    0,    0,  105,  107,    0,    0,   24,    0,
   39,    0,  102,    0,    0,  104,   27,    0,   15,   14,
  110,    0,  109,  103,  111,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   73,    0,    0,    0,
   54,    0,    0,    0,   37,    0,    0,    0,   53,   83,
  115,  106,  108,   75,    0,    0,    0,    0,    0,   92,
    0,    0,    0,    0,    0,   91,    0,    0,    0,    0,
    0,   86,    0,    0,   56,    0,    0,   89,   40,   41,
   42,   78,    0,    0,    0,    0,    0,  101,   94,   44,
   43,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   57,    0,    0,    0,
   90,    0,   51,   52,   76,    0,    0,   63,   98,    0,
   95,    0,    0,   62,    0,   74,    0,   59,   60,    0,
    0,    0,    0,    0,    0,    0,    0,   55,    0,   77,
    0,    0,    0,    0,    0,    0,   71,   72,   64,   68,
    0,    0,
};
const short yydgoto[] =
	{                                       1,
   28,  184,   24,   81,  152,   66,   16,   17,   18,   19,
   20,   56,   63,   33,   57,   78,  106,  107,  108,  109,
  155,  135,  185,  167,  199,  131,  182,  110,  160,  126,
   67,   68,   76,   82,   72,  112,   83,
};
const short yysindex[] =
	{                                      0,
  -10,    3, -288, -278, -236,  -90, -277, -269, -288, -288,
  -81, -288,  -13,    0,   40,   42,   48,   53,   55,    0,
    0,    0,    0, -233,    0,    0,    0,    0, -288, -241,
 -254,    7,    0,    0,    0, -233, -233,    7, -233, -288,
    0,    0,    0,    0,    0,    0, -233,    0,    0,    0,
    0, -288,    0,    7,    7,    0,  -56, -253, -233, -233,
    0,    0,  -82,    7,    0,    0, -119,   62,    0,    7,
    0, -247,    0,   62,    7,    0,    0, 1147,    0,    0,
    0,  -43,    0,    0,    0,  -48, -288, -194, -212, -288,
 -221, -114, -288, -288, -288, -201,    0, -236, -288, -288,
    0, -181, -288, -225,    0,    7,    7,    7,    0,    0,
    0,    0,    0,    0, -233, -236, -224, -233, -288,    0,
 -288, -273, -222,    7, -233,    0, -233, -233, -233,  -41,
 -288,    0, -233,  -58,    0, -173, -233,    0,    0,    0,
    0,    0, -288, -233,  -57, -288, -233,    0,    0,    0,
    0, -196, -118, -288, -111, -288, -233, -288, -267,  -86,
 -240,    7, -233, -288, -288,    7,    0, -233, -233, -238,
    0,    7,    0,    0,    0, -264, -233,    0,    0, -233,
    0, 1003, -235,    0, -103,    0,    7,    0,    0, -288,
 -288, -288, -288, -174, -288, -236, -236,    0,    7,    0,
 -233, -233, -233, -264, -170, -233,    0,    0,    0,    0,
 -288, -233,};
const short yyrindex[] =
	{                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   82,    0,    0,    0,    0,    0,    0,
   -8,  -80,    0,    0,    0,   83,  -29, -249,   85,    0,
    0,    0,    0,    0,    0,    0,   -2,    0,    0,    0,
    0,    0,    0, 1096, 1096,    0,    0,    0,   86,    1,
    0,    0,    0, 1183,    0,    0,    0,    0,    0,  -80,
    0,    0,    0,    0, -115,    0,    0,    0,    0,    0,
    0,   87,    0,    0,    0,    0,    0,    0,  294,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1183, 1183, 1183,    0,    0,
    0,    0,    0,    0,  337,    0,    0,  384,    0,    0,
    0,    0,    0,  -72,  424,    0,  475,  522,  566,    0,
    0,    0,  613,    0,    0,    0,  660,    0,    0,    0,
    0,    0,  702,  753,    0,    0,  106,    0,    0,    0,
    0,  789,    0,    0,    0,    0,  825,    0,  148,    0,
    0, 1025,  867,    0,    0,  -91,    0,  914,  195,    0,
    0,  -72,    0,    0,    0, -109,  -85,    0,    0,  246,
    0,    0,    0,    0,    0,    0, 1025,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  -91,    0,
   -3,   19,   27,   44,    0,   54,    0,    0,    0,    0,
    0,   66,};
const short yygindex[] =
	{                                      0,
  -67, -106, 1156,    0,    0,   98,    0,    0,    0,    0,
    0, 1411,    0,   37,    0,    0,    0,    0,    0,  -78,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  -59,
    0,   35,   31,    0,    0,    0,   24,
};
#define YYTABLESIZE 1610
const short yytable[] =
	{                                      14,
  151,   35,  154,  158,  162,   73,   65,   28,  124,  114,
   33,  166,   21,   58,   52,  111,   54,  170,    4,   23,
  183,  198,  114,   48,   49,   50,  146,   53,   66,   25,
  132,   34,   32,  114,   23,   51,   67,   61,  171,   35,
   46,   38,   69,   46,  114,   26,   27,   40,  142,   41,
   35,   42,  114,   58,   65,   65,   28,   43,  114,   33,
   79,   80,   44,   70,   45,   55,   64,  173,  174,   23,
  179,   75,  188,  189,   46,   69,  114,   66,  116,  117,
  119,  130,  136,  138,  143,   67,  148,  156,  161,  205,
  211,   17,   22,   36,   23,   16,  116,  210,   15,   70,
  172,   74,   58,  187,   84,  113,    0,    0,    0,    0,
    0,    0,   70,    0,    0,   97,   35,    0,    0,    0,
    0,   65,   28,    0,   69,   33,    0,   22,  207,  208,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   66,    0,    0,    0,    0,    0,    0,
    0,   67,    4,    0,    0,    0,  114,   99,  190,  164,
    0,   58,    0,    0,   97,    0,   29,  120,   58,  121,
  114,  191,    0,    0,   29,  165,  114,   58,   70,    0,
    0,  122,  123,  114,  192,   61,  193,  194,   65,   46,
   69,  195,  114,   23,    0,    0,  114,  121,  114,  114,
  196,   61,  197,  114,   96,    0,   99,   30,   31,  122,
  123,  114,  114,    0,  114,   30,   31,  114,  114,    0,
    0,    0,    0,  114,  114,    0,    0,    0,    0,    0,
   97,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    2,    0,    0,   35,   46,
   46,    0,    3,   96,   28,  100,    0,   33,   65,    0,
    0,    4,    0,    5,   79,   80,  150,    0,    0,    6,
    0,   65,   99,    0,    7,    8,    0,    9,    0,    0,
   66,    0,    0,   10,   65,    0,   65,   65,   67,   35,
   35,   65,   11,   66,   12,   28,   28,   13,   33,   33,
   65,   67,   65,   81,  100,   58,   66,    0,   66,   66,
    0,    0,    0,   66,   67,   70,   67,   67,   58,   96,
    0,   67,   66,    0,   66,    0,    0,   69,   70,    0,
   67,   58,   67,   58,   58,    0,    0,    0,   58,    0,
   69,   70,    0,   70,   70,    0,   45,   58,   70,   58,
    0,    0,   81,   69,    0,   69,   69,   70,    0,   70,
   69,   97,    0,   97,   97,   97,    0,   97,    0,   69,
  100,   69,    0,   97,    0,   97,    0,    0,   97,    0,
   97,   97,    0,   97,   97,   97,   97,    0,    0,   97,
    0,    0,    0,   46,   97,   45,   97,    0,   97,    0,
    0,   97,   97,   99,    0,   99,   99,   99,    0,   99,
    0,    0,    0,    0,    0,   99,    0,   99,   81,    0,
   99,    0,   99,   99,    0,   99,   99,   99,   99,    0,
    0,   99,    0,   93,    0,    0,   99,    0,   99,    0,
   99,    0,   46,   99,   99,    0,    0,    0,    0,    0,
   96,    0,   96,   96,   96,    0,   96,    0,    0,    0,
    0,   45,   96,    0,   96,    0,    0,   96,    0,   96,
   96,    0,   96,   96,   96,   96,    0,    0,   96,    0,
    0,    0,   93,   96,   84,   96,    0,   96,    0,    0,
   96,   96,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  100,    0,  100,  100,  100,    0,  100,   46,    0,
    0,    0,    0,  100,    0,  100,    0,    0,  100,    0,
  100,  100,    0,  100,  100,  100,  100,    0,    0,  100,
    0,   47,    0,   84,  100,    0,  100,    0,  100,    0,
    0,  100,  100,    0,    0,    0,    0,    0,   93,   81,
    0,   81,   81,   81,    0,   81,    0,    0,    0,    0,
    0,   81,    0,   81,    0,    0,   81,    0,   81,   81,
    0,   81,   81,   81,   81,   85,    0,   81,    0,    0,
   47,    0,   81,    0,   81,    0,   81,    0,    0,    0,
   81,    0,   45,    0,   45,   45,   45,    0,   45,   84,
    0,    0,    0,    0,   45,    0,   45,    0,    0,   45,
    0,   45,   45,    0,   45,   45,   45,   45,    0,    0,
   45,    0,   48,    0,   85,   45,    0,   45,    0,   45,
    0,    0,    0,   45,    0,    0,    0,    0,    0,   46,
    0,   46,   46,   46,    0,   46,   47,    0,    0,    0,
    0,   46,    0,   46,    0,    0,   46,    0,   46,   46,
    0,   46,   46,   46,   46,    0,    0,   46,    0,   88,
    0,   48,   46,    0,   46,    0,   46,    0,    0,   93,
   46,   93,   93,   93,    0,   93,    0,    0,    0,    0,
   85,   93,    0,   93,    0,    0,   93,    0,   93,   93,
    0,   93,   93,   93,   93,    0,    0,   93,    0,    0,
    0,   80,   93,    0,   93,    0,   93,    0,   88,    0,
   93,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   84,    0,   84,   84,   84,    0,   84,   48,    0,    0,
    0,    0,   84,    0,   84,    0,    0,   84,    0,   84,
   84,    0,   84,   84,   84,   84,    0,    0,   84,    0,
   80,    0,   82,   84,    0,   84,    0,   84,    0,    0,
    0,   84,    0,    0,    0,    0,    0,   47,    0,   47,
   47,   47,    0,   47,   88,    0,    0,    0,    0,   47,
    0,   47,    0,    0,   47,    0,   47,   47,   50,   47,
   47,   47,   47,    0,    0,   47,    0,    0,    0,    0,
   47,   82,   47,    0,   47,    0,    0,    0,   47,    0,
    0,   85,    0,   85,   85,   85,   80,   85,    0,    0,
    0,    0,    0,   85,   79,   85,    0,    0,   85,    0,
   85,   85,    0,   85,   85,   85,   85,   50,    0,   85,
    0,    0,    0,    0,   85,    0,   85,    0,   85,    0,
    0,    0,   85,    0,    0,    0,    0,    0,   48,    0,
   48,   48,   48,    0,   48,    0,   49,   82,    0,    0,
   48,    0,   48,   79,    0,   48,    0,   48,   48,    0,
   48,   48,   48,   48,    0,    0,   48,    0,    0,    0,
    0,   48,    0,   48,    0,   48,    0,    0,    0,   48,
    0,    0,    0,   50,    0,   88,    0,   88,   88,   88,
    0,   88,    0,   87,    0,   49,    0,   88,    0,   88,
    0,    0,   88,    0,   88,   88,    0,   88,   88,   88,
   88,    0,    0,   88,    0,    0,    0,    0,   88,   79,
   88,    0,   88,    0,    0,    0,   88,   80,    0,   80,
   80,   80,    0,   80,    0,    0,    0,    0,    0,   80,
    0,   80,   87,    0,   80,    0,   80,   80,    0,   80,
   80,   80,   80,    0,    0,   80,    0,    0,    0,    0,
   80,   49,   80,    0,   80,    0,    0,    0,   80,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   82,    0,
   82,   82,   82,    0,   82,    0,    0,    0,    0,    0,
   82,    0,   82,    0,    0,   82,    0,   82,   82,    0,
   82,   82,   82,   82,    0,    0,   82,    0,   87,    0,
    0,   82,    0,   82,   50,   82,   50,   50,   50,   82,
   50,    0,    0,    0,    0,    0,   50,    0,   50,    0,
    0,   50,    0,   50,   50,    0,   50,   50,   50,   50,
    0,    0,   50,    0,    0,    0,    0,   50,    0,   50,
   79,   50,   79,   79,   79,   50,   79,    0,    0,    0,
    0,    0,   79,    0,   79,    0,    0,   79,    0,   79,
   79,    0,   79,   79,   79,   79,    0,    0,   79,    0,
    0,    0,    0,   79,    0,   79,    0,   79,    0,    0,
    0,   79,   49,    0,   49,   49,   49,  186,   49,    0,
    0,    0,    0,    0,   49,    0,   49,    0,    0,   49,
    0,   49,   49,    0,   49,   49,   49,   49,    0,  114,
   49,    0,    0,    0,    0,   49,    0,   49,    0,   49,
    0,    0,    0,   49,   36,   37,    0,   39,    0,   87,
    0,   87,   87,   87,    0,   87,    0,    0,    0,    0,
    0,   87,    0,   87,   47,    0,   87,    0,   87,   87,
    0,   87,   87,   87,   87,   59,    0,   87,    0,    0,
    0,    0,   87,    0,   87,    0,   87,   60,    0,    0,
   87,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  114,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  115,    0,    0,  118,    0,  125,  127,  128,
  129,    0,    0,    0,  133,  134,    0,    0,  137,    0,
    0,   88,   89,    0,    0,    0,    0,    0,    0,    0,
   91,  105,   92,    0,  144,   93,  145,  147,   95,    0,
    0,    0,   98,  114,  114,    0,  153,    0,    0,    0,
    0,    0,  114,  102,  114,  103,    0,  114,  157,  104,
  114,  159,    0,    0,  114,    0,    0,  114,    0,  163,
    0,  168,    0,  169,    0,  114,    0,  114,    0,  176,
  177,  114,    0,    0,    0,  180,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  201,  202,  203,  204,    0,
  206,  114,  114,  114,  114,  114,    0,  114,    0,    0,
    0,    0,    0,  114,    0,  114,  212,  114,  114,    0,
  114,  114,    0,  114,  114,  114,  114,    0,    0,  114,
    0,    0,    0,  114,  114,  114,  114,    0,  114,    0,
  114,  114,  114,  114,  114,    0,    0,    0,    0,  114,
    0,  114,   86,  114,   87,   88,   89,    0,   90,    0,
    0,    0,    0,    0,   91,    0,   92,    0,    0,   93,
    0,   94,   95,    0,   96,   97,   98,   99,    0,    0,
  100,    0,    0,    0,    0,  101,    0,  102,  114,  103,
  114,  114,  114,  104,  114,    0,    0,    0,   58,    0,
  114,    0,  114,    0,    0,  114,    0,  114,  114,    0,
  114,  114,  114,  114,   61,   62,  114,    0,    0,    0,
    0,  114,    0,  114,   71,  114,    0,    0,    0,  114,
   77,    0,    0,    0,    0,   85,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  139,  140,  141,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  149,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  175,    0,    0,    0,  178,    0,    0,    0,
    0,    0,  181,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  200,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  209,
};
const short yycheck[] =
	{                                      10,
   42,   10,   61,   61,  123,  125,   10,   10,  123,  125,
   10,  123,   10,  123,  269,   59,   10,  285,  272,  308,
  285,  125,  272,  265,  266,  267,  300,  282,   10,  308,
   98,  309,  123,  125,  308,  277,   10,  123,  125,  309,
  308,  123,  125,  308,  125,  282,  283,   61,  116,   10,
   59,   10,  125,   10,  308,   59,   59,   10,  308,   59,
  308,  309,   10,   10,   10,   59,  123,  308,  309,  308,
  309,   10,  308,  309,  308,   10,  125,   59,  273,  292,
  302,  283,  264,  309,  309,   59,  309,  261,  285,  264,
  261,   10,   10,  123,   10,   10,   10,  204,    1,   63,
  160,   67,   59,  182,   74,   82,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   -1,   -1,   10,  125,   -1,   -1,   -1,
   -1,  125,  125,   -1,   59,  125,   -1,  125,  196,  197,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  125,  272,   -1,   -1,   -1,  272,   10,  262,  271,
   -1,  271,   -1,   -1,   59,   -1,  257,  282,  125,  284,
  262,  275,   -1,   -1,  257,  287,  257,  287,  125,   -1,
   -1,  296,  297,  275,  288,  271,  290,  291,  308,  308,
  125,  295,  308,  308,   -1,   -1,  288,  284,  290,  291,
  304,  287,  306,  295,   10,   -1,   59,  298,  299,  296,
  297,  284,  304,   -1,  306,  298,  299,  298,  299,   -1,
   -1,   -1,   -1,  296,  297,   -1,   -1,   -1,   -1,   -1,
  125,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  256,   -1,   -1,  257,  308,
  308,   -1,  263,   59,  257,   10,   -1,  257,  262,   -1,
   -1,  272,   -1,  274,  308,  309,  308,   -1,   -1,  280,
   -1,  275,  125,   -1,  285,  286,   -1,  288,   -1,   -1,
  262,   -1,   -1,  294,  288,   -1,  290,  291,  262,  298,
  299,  295,  303,  275,  305,  298,  299,  308,  298,  299,
  304,  275,  306,   10,   59,  262,  288,   -1,  290,  291,
   -1,   -1,   -1,  295,  288,  262,  290,  291,  275,  125,
   -1,  295,  304,   -1,  306,   -1,   -1,  262,  275,   -1,
  304,  288,  306,  290,  291,   -1,   -1,   -1,  295,   -1,
  275,  288,   -1,  290,  291,   -1,   10,  304,  295,  306,
   -1,   -1,   59,  288,   -1,  290,  291,  304,   -1,  306,
  295,  256,   -1,  258,  259,  260,   -1,  262,   -1,  304,
  125,  306,   -1,  268,   -1,  270,   -1,   -1,  273,   -1,
  275,  276,   -1,  278,  279,  280,  281,   -1,   -1,  284,
   -1,   -1,   -1,   10,  289,   59,  291,   -1,  293,   -1,
   -1,  296,  297,  256,   -1,  258,  259,  260,   -1,  262,
   -1,   -1,   -1,   -1,   -1,  268,   -1,  270,  125,   -1,
  273,   -1,  275,  276,   -1,  278,  279,  280,  281,   -1,
   -1,  284,   -1,   10,   -1,   -1,  289,   -1,  291,   -1,
  293,   -1,   59,  296,  297,   -1,   -1,   -1,   -1,   -1,
  256,   -1,  258,  259,  260,   -1,  262,   -1,   -1,   -1,
   -1,  125,  268,   -1,  270,   -1,   -1,  273,   -1,  275,
  276,   -1,  278,  279,  280,  281,   -1,   -1,  284,   -1,
   -1,   -1,   59,  289,   10,  291,   -1,  293,   -1,   -1,
  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  256,   -1,  258,  259,  260,   -1,  262,  125,   -1,
   -1,   -1,   -1,  268,   -1,  270,   -1,   -1,  273,   -1,
  275,  276,   -1,  278,  279,  280,  281,   -1,   -1,  284,
   -1,   10,   -1,   59,  289,   -1,  291,   -1,  293,   -1,
   -1,  296,  297,   -1,   -1,   -1,   -1,   -1,  125,  256,
   -1,  258,  259,  260,   -1,  262,   -1,   -1,   -1,   -1,
   -1,  268,   -1,  270,   -1,   -1,  273,   -1,  275,  276,
   -1,  278,  279,  280,  281,   10,   -1,  284,   -1,   -1,
   59,   -1,  289,   -1,  291,   -1,  293,   -1,   -1,   -1,
  297,   -1,  256,   -1,  258,  259,  260,   -1,  262,  125,
   -1,   -1,   -1,   -1,  268,   -1,  270,   -1,   -1,  273,
   -1,  275,  276,   -1,  278,  279,  280,  281,   -1,   -1,
  284,   -1,   10,   -1,   59,  289,   -1,  291,   -1,  293,
   -1,   -1,   -1,  297,   -1,   -1,   -1,   -1,   -1,  256,
   -1,  258,  259,  260,   -1,  262,  125,   -1,   -1,   -1,
   -1,  268,   -1,  270,   -1,   -1,  273,   -1,  275,  276,
   -1,  278,  279,  280,  281,   -1,   -1,  284,   -1,   10,
   -1,   59,  289,   -1,  291,   -1,  293,   -1,   -1,  256,
  297,  258,  259,  260,   -1,  262,   -1,   -1,   -1,   -1,
  125,  268,   -1,  270,   -1,   -1,  273,   -1,  275,  276,
   -1,  278,  279,  280,  281,   -1,   -1,  284,   -1,   -1,
   -1,   10,  289,   -1,  291,   -1,  293,   -1,   59,   -1,
  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  256,   -1,  258,  259,  260,   -1,  262,  125,   -1,   -1,
   -1,   -1,  268,   -1,  270,   -1,   -1,  273,   -1,  275,
  276,   -1,  278,  279,  280,  281,   -1,   -1,  284,   -1,
   59,   -1,   10,  289,   -1,  291,   -1,  293,   -1,   -1,
   -1,  297,   -1,   -1,   -1,   -1,   -1,  256,   -1,  258,
  259,  260,   -1,  262,  125,   -1,   -1,   -1,   -1,  268,
   -1,  270,   -1,   -1,  273,   -1,  275,  276,   10,  278,
  279,  280,  281,   -1,   -1,  284,   -1,   -1,   -1,   -1,
  289,   59,  291,   -1,  293,   -1,   -1,   -1,  297,   -1,
   -1,  256,   -1,  258,  259,  260,  125,  262,   -1,   -1,
   -1,   -1,   -1,  268,   10,  270,   -1,   -1,  273,   -1,
  275,  276,   -1,  278,  279,  280,  281,   59,   -1,  284,
   -1,   -1,   -1,   -1,  289,   -1,  291,   -1,  293,   -1,
   -1,   -1,  297,   -1,   -1,   -1,   -1,   -1,  256,   -1,
  258,  259,  260,   -1,  262,   -1,   10,  125,   -1,   -1,
  268,   -1,  270,   59,   -1,  273,   -1,  275,  276,   -1,
  278,  279,  280,  281,   -1,   -1,  284,   -1,   -1,   -1,
   -1,  289,   -1,  291,   -1,  293,   -1,   -1,   -1,  297,
   -1,   -1,   -1,  125,   -1,  256,   -1,  258,  259,  260,
   -1,  262,   -1,   10,   -1,   59,   -1,  268,   -1,  270,
   -1,   -1,  273,   -1,  275,  276,   -1,  278,  279,  280,
  281,   -1,   -1,  284,   -1,   -1,   -1,   -1,  289,  125,
  291,   -1,  293,   -1,   -1,   -1,  297,  256,   -1,  258,
  259,  260,   -1,  262,   -1,   -1,   -1,   -1,   -1,  268,
   -1,  270,   59,   -1,  273,   -1,  275,  276,   -1,  278,
  279,  280,  281,   -1,   -1,  284,   -1,   -1,   -1,   -1,
  289,  125,  291,   -1,  293,   -1,   -1,   -1,  297,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  256,   -1,
  258,  259,  260,   -1,  262,   -1,   -1,   -1,   -1,   -1,
  268,   -1,  270,   -1,   -1,  273,   -1,  275,  276,   -1,
  278,  279,  280,  281,   -1,   -1,  284,   -1,  125,   -1,
   -1,  289,   -1,  291,  256,  293,  258,  259,  260,  297,
  262,   -1,   -1,   -1,   -1,   -1,  268,   -1,  270,   -1,
   -1,  273,   -1,  275,  276,   -1,  278,  279,  280,  281,
   -1,   -1,  284,   -1,   -1,   -1,   -1,  289,   -1,  291,
  256,  293,  258,  259,  260,  297,  262,   -1,   -1,   -1,
   -1,   -1,  268,   -1,  270,   -1,   -1,  273,   -1,  275,
  276,   -1,  278,  279,  280,  281,   -1,   -1,  284,   -1,
   -1,   -1,   -1,  289,   -1,  291,   -1,  293,   -1,   -1,
   -1,  297,  256,   -1,  258,  259,  260,  125,  262,   -1,
   -1,   -1,   -1,   -1,  268,   -1,  270,   -1,   -1,  273,
   -1,  275,  276,   -1,  278,  279,  280,  281,   -1,  125,
  284,   -1,   -1,   -1,   -1,  289,   -1,  291,   -1,  293,
   -1,   -1,   -1,  297,    9,   10,   -1,   12,   -1,  256,
   -1,  258,  259,  260,   -1,  262,   -1,   -1,   -1,   -1,
   -1,  268,   -1,  270,   29,   -1,  273,   -1,  275,  276,
   -1,  278,  279,  280,  281,   40,   -1,  284,   -1,   -1,
   -1,   -1,  289,   -1,  291,   -1,  293,   52,   -1,   -1,
  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  125,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   87,   -1,   -1,   90,   -1,   92,   93,   94,
   95,   -1,   -1,   -1,   99,  100,   -1,   -1,  103,   -1,
   -1,  259,  260,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  268,  125,  270,   -1,  119,  273,  121,  122,  276,   -1,
   -1,   -1,  280,  259,  260,   -1,  131,   -1,   -1,   -1,
   -1,   -1,  268,  291,  270,  293,   -1,  273,  143,  297,
  276,  146,   -1,   -1,  280,   -1,   -1,  125,   -1,  154,
   -1,  156,   -1,  158,   -1,  291,   -1,  293,   -1,  164,
  165,  297,   -1,   -1,   -1,  170,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  190,  191,  192,  193,   -1,
  195,  256,  257,  258,  259,  260,   -1,  262,   -1,   -1,
   -1,   -1,   -1,  268,   -1,  270,  211,  272,  273,   -1,
  275,  276,   -1,  278,  279,  280,  281,   -1,   -1,  284,
   -1,   -1,   -1,  288,  289,  290,  291,   -1,  293,   -1,
  295,  296,  297,  298,  299,   -1,   -1,   -1,   -1,  304,
   -1,  306,  256,  308,  258,  259,  260,   -1,  262,   -1,
   -1,   -1,   -1,   -1,  268,   -1,  270,   -1,   -1,  273,
   -1,  275,  276,   -1,  278,  279,  280,  281,   -1,   -1,
  284,   -1,   -1,   -1,   -1,  289,   -1,  291,  256,  293,
  258,  259,  260,  297,  262,   -1,   -1,   -1,   38,   -1,
  268,   -1,  270,   -1,   -1,  273,   -1,  275,  276,   -1,
  278,  279,  280,  281,   54,   55,  284,   -1,   -1,   -1,
   -1,  289,   -1,  291,   64,  293,   -1,   -1,   -1,  297,
   70,   -1,   -1,   -1,   -1,   75,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  106,  107,  108,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  124,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  162,   -1,   -1,   -1,  166,   -1,   -1,   -1,
   -1,   -1,  172,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  187,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  199,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 309
#if YYDEBUG
const char * const yyname[] =
	{
"end-of-file",0,0,0,0,0,0,0,0,0,"'\\n'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,"'*'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"';'",0,"'='",0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"ACCESS","ALIAS","AUTO","BLOCK","CA","CERT","CHROOT","CLIENT","COMBINED",
"COMMON","CONDENSED","DEFAULT","FACILITY","FASTCGI","FOR_HOST","INCLUDE",
"INDEX","IPV6","KEY","LANG","LEGACY","LISTEN","LOCATION","LOG","OCSP","OFF",
"ON","PARAM","PORT","PREFORK","PROTO","PROTOCOLS","PROXY","RELAY_TO","REQUIRE",
"RETURN","ROOT","SERVER","SNI","SOCKET","STRIP","STYLE","SYSLOG","TCP","TOEXT",
"TYPE","TYPES","USE_TLS","USER","VERIFYNAME","ERROR","STRING","NUM",
};
const char * const yyrule[] =
	{"$accept : conf",
"conf :",
"conf : conf include '\\n'",
"conf : conf '\\n'",
"conf : conf varset '\\n'",
"conf : conf option '\\n'",
"conf : conf vhost '\\n'",
"conf : conf types '\\n'",
"conf : conf error '\\n'",
"include : INCLUDE STRING",
"bool : ON",
"bool : OFF",
"string : string STRING",
"string : STRING",
"numberstring : NUM",
"numberstring : STRING",
"varset : STRING '=' string",
"option : CHROOT string",
"option : IPV6 bool",
"option : log",
"option : PORT NUM",
"option : PREFORK NUM",
"option : PROTOCOLS string",
"option : USER string",
"log : LOG '{' optnl logopts '}'",
"log : LOG logopt",
"logopts :",
"logopts : logopts logopt optnl",
"logopt : ACCESS string",
"logopt : STYLE COMMON",
"logopt : STYLE COMBINED",
"logopt : STYLE CONDENSED",
"logopt : STYLE LEGACY",
"logopt : SYSLOG FACILITY string",
"logopt : SYSLOG OFF",
"logopt : SYSLOG",
"$$1 :",
"vhost : SERVER string $$1 '{' optnl servbody '}'",
"vhost : error '}'",
"servbody :",
"servbody : servbody servopt optnl",
"servbody : servbody location optnl",
"servbody : servbody proxy optnl",
"listen_addr : '*'",
"listen_addr : STRING",
"servopt : ALIAS string",
"servopt : CERT string",
"servopt : KEY string",
"servopt : OCSP string",
"servopt : PARAM string '=' string",
"servopt : LISTEN ON listen_addr",
"servopt : LISTEN ON listen_addr PORT STRING",
"servopt : LISTEN ON listen_addr PORT NUM",
"servopt : locopt",
"$$2 :",
"proxy : PROXY $$2 proxy_matches '{' optnl proxy_opts '}'",
"proxy_matches :",
"proxy_matches : proxy_matches proxy_match",
"proxy_port :",
"proxy_port : PORT STRING",
"proxy_port : PORT NUM",
"proxy_match : PROTO string",
"proxy_match : FOR_HOST string proxy_port",
"proxy_opts :",
"proxy_opts : proxy_opts proxy_opt optnl",
"proxy_opt : CERT string",
"proxy_opt : KEY string",
"proxy_opt : PROTOCOLS string",
"proxy_opt : RELAY_TO string proxy_port",
"proxy_opt : REQUIRE CLIENT CA string",
"proxy_opt : SNI string",
"proxy_opt : USE_TLS bool",
"proxy_opt : VERIFYNAME bool",
"$$3 :",
"location : LOCATION $$3 string '{' optnl locopts '}'",
"location : error '}'",
"locopts :",
"locopts : locopts locopt optnl",
"locopt : AUTO INDEX bool",
"locopt : BLOCK RETURN NUM string",
"locopt : BLOCK RETURN NUM",
"locopt : BLOCK",
"locopt : DEFAULT TYPE string",
"locopt : fastcgi",
"locopt : INDEX string",
"locopt : LANG string",
"locopt : LOG bool",
"locopt : REQUIRE CLIENT CA string",
"locopt : ROOT string",
"locopt : STRIP NUM",
"fastcgi : FASTCGI '{' optnl fastcgiopts '}'",
"fastcgi : FASTCGI fastcgiopt",
"fastcgi : FASTCGI OFF",
"fastcgi : FASTCGI string",
"fastcgiopts :",
"fastcgiopts : fastcgiopts fastcgiopt optnl",
"fastcgiopt : PARAM string '=' string",
"fastcgiopt : SOCKET string",
"fastcgiopt : SOCKET TCP string PORT NUM",
"fastcgiopt : SOCKET TCP string",
"fastcgiopt : SOCKET TCP string PORT string",
"fastcgiopt : STRIP NUM",
"types : TYPES '{' optnl mediaopts_l '}'",
"mediaopts_l : mediaopts_l mediaoptsl nl",
"mediaopts_l : mediaoptsl nl",
"$$4 :",
"mediaoptsl : STRING $$4 medianames_l optsemicolon",
"mediaoptsl : include",
"medianames_l : medianames_l medianamesl",
"medianames_l : medianamesl",
"medianamesl : numberstring",
"nl : '\\n' optnl",
"optnl : '\\n' optnl",
"optnl : ';' optnl",
"optnl :",
"optsemicolon : ';'",
"optsemicolon :",
};
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
/* LINTUSED */
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
unsigned int yystacksize;
int yyparse(void);
#line 648 "parse.y"

static const struct keyword {
	const char *word;
	int token;
} keywords[] = {
	/* these MUST be sorted */
	{"access", ACCESS},
	{"alias", ALIAS},
	{"auto", AUTO},
	{"block", BLOCK},
	{"ca", CA},
	{"cert", CERT},
	{"chroot", CHROOT},
	{"client", CLIENT},
	{"combined", COMBINED},
	{"common", COMMON},
	{"condensed", CONDENSED},
	{"default", DEFAULT},
	{"facility", FACILITY},
	{"fastcgi", FASTCGI},
	{"for-host", FOR_HOST},
	{"include", INCLUDE},
	{"index", INDEX},
	{"ipv6", IPV6},
	{"key", KEY},
	{"lang", LANG},
	{"legacy", LEGACY},
	{"listen", LISTEN},
	{"location", LOCATION},
	{"log", LOG},
	{"ocsp", OCSP},
	{"off", OFF},
	{"on", ON},
	{"param", PARAM},
	{"port", PORT},
	{"prefork", PREFORK},
	{"proto", PROTO},
	{"protocols", PROTOCOLS},
	{"proxy", PROXY},
	{"relay-to", RELAY_TO},
	{"require", REQUIRE},
	{"return", RETURN},
	{"root", ROOT},
	{"server", SERVER},
	{"sni", SNI},
	{"socket", SOCKET},
	{"strip", STRIP},
	{"style", STYLE},
	{"syslog", SYSLOG},
	{"tcp", TCP},
	{"to-ext", TOEXT},
	{"type", TYPE},
	{"types", TYPES},
	{"use-tls", USE_TLS},
	{"user", USER},
	{"verifyname", VERIFYNAME},
};

void
yyerror(const char *msg, ...)
{
	va_list ap;

	file->errors++;

	va_start(ap, msg);
	fprintf(stderr, "%s:%d error: ", config_path, yylval.lineno);
	vfprintf(stderr, msg, ap);
	fprintf(stderr, "\n");
	va_end(ap);
}

void
yywarn(const char *msg, ...)
{
	va_list ap;

	va_start(ap, msg);
	fprintf(stderr, "%s:%d warning: ", config_path, yylval.lineno);
	vfprintf(stderr, msg, ap);
	fprintf(stderr, "\n");
	va_end(ap);
}

int
kw_cmp(const void *k, const void *e)
{
	return strcmp(k, ((struct keyword *)e)->word);
}

int
lookup(char *s)
{
	const struct keyword	*p;

	p = bsearch(s, keywords, sizeof(keywords)/sizeof(keywords[0]),
	    sizeof(keywords[0]), kw_cmp);

	if (p)
		return p->token;
	else
		return STRING;
}

#define START_EXPAND	1
#define DONE_EXPAND	2

static int	expanding;

int
igetc(void)
{
	int	c;

	while (1) {
		if (file->ungetpos > 0)
			c = file->ungetbuf[--file->ungetpos];
		else
			c = getc(file->stream);

		if (c == START_EXPAND)
			expanding = 1;
		else if (c == DONE_EXPAND)
			expanding = 0;
		else
			break;
	}
	return c;
}

int
lgetc(int quotec)
{
	int		c, next;

	if (quotec) {
		if ((c = igetc()) == EOF) {
			yyerror("reached end of file while parsing "
			    "quoted string");
			if (file == topfile || popfile() == EOF)
				return EOF;
			return quotec;
		}
		return c;
	}

	while ((c = igetc()) == '\\') {
		next = igetc();
		if (next != '\n') {
			c = next;
			break;
		}
		yylval.lineno = file->lineno;
		file->lineno++;
	}

	if (c == EOF) {
		/*
		 * Fake EOL when hit EOF for the first time. This gets line
		 * count right if last line in included file is syntactically
		 * invalid and has no newline.
		 */
		if (file->eof_reached == 0) {
			file->eof_reached = 1;
			return '\n';
		}
		while (c == EOF) {
			if (file == topfile || popfile() == EOF)
				return EOF;
			c = igetc();
		}
	}
	return c;
}

void
lungetc(int c)
{
	if (c == EOF)
		return;

	if (file->ungetpos >= file->ungetsize) {
		void *p = reallocarray(file->ungetbuf, file->ungetsize, 2);
		if (p == NULL)
			fatal("lungetc");
		file->ungetbuf = p;
		file->ungetsize *= 2;
	}
	file->ungetbuf[file->ungetpos++] = c;
}

int
findeol(void)
{
	int	c;

	/* Skip to either EOF or the first real EOL. */
	while (1) {
		c = lgetc(0);
		if (c == '\n') {
			file->lineno++;
			break;
		}
		if (c == EOF)
			break;
	}
	return ERROR;
}

int
yylex(void)
{
	char	 buf[8096];
	char	*p, *val;
	int	 quotec, next, c;
	int	 token;

top:
	p = buf;
	while ((c = lgetc(0)) == ' ' || c == '\t')
		; /* nothing */

	yylval.lineno = file->lineno;
	if (c == '#')
		while ((c = lgetc(0)) != '\n' && c != EOF)
			; /* nothing */
	if (c == '$' && !expanding) {
		while (1) {
			if ((c = lgetc(0)) == EOF)
				return 0;
			if (p + 1 >= buf + sizeof(buf) -1) {
				yyerror("string too long");
				return findeol();
			}
			if (isalnum(c) || c == '_') {
				*p++ = c;
				continue;
			}
			*p = '\0';
			lungetc(c);
			break;
		}
		val = symget(buf);
		if (val == NULL) {
			yyerror("macro `%s' not defined", buf);
			return findeol();
		}
		yylval.v.string = xstrdup(val);
		return STRING;
	}
	if (c == '@' && !expanding) {
		while (1) {
			if ((c = lgetc(0)) == EOF)
				return 0;

			if (p + 1 >= buf + sizeof(buf) - 1) {
				yyerror("string too long");
				return findeol();
			}
			if (isalnum(c) || c == '_') {
				*p++ = c;
				continue;
			}
			*p = '\0';
			lungetc(c);
			break;
		}
		val = symget(buf);
		if (val == NULL) {
			yyerror("macro '%s' not defined", buf);
			return findeol();
		}
		p = val + strlen(val) - 1;
		lungetc(DONE_EXPAND);
		while (p >= val) {
			lungetc(*p);
			p--;
		}
		lungetc(START_EXPAND);
		goto top;
	}

	switch (c) {
	case '\'':
	case '"':
		quotec = c;
		while (1) {
			if ((c = lgetc(quotec)) == EOF)
				return 0;
			if (c == '\n') {
				file->lineno++;
				continue;
			} else if (c == '\\') {
				if ((next = lgetc(quotec)) == EOF)
					return (0);
				if (next == quotec || next == ' ' ||
				    next == '\t')
					c = next;
				else if (next == '\n') {
					file->lineno++;
					continue;
				} else
					lungetc(next);
			} else if (c == quotec) {
				*p = '\0';
				break;
			} else if (c == '\0') {
				yyerror("invalid syntax");
				return findeol();
			}
			if (p + 1 >= buf + sizeof(buf) - 1) {
				yyerror("string too long");
				return findeol();
			}
			*p++ = c;
		}
		yylval.v.string = strdup(buf);
		if (yylval.v.string == NULL)
			fatal("yylex: strdup");
		return STRING;
	}

#define allowed_to_end_number(x) \
	(isspace(x) || x == ')' || x ==',' || x == '/' || x == '}' || x == '=')

	if (c == '-' || isdigit(c)) {
		do {
			*p++ = c;
			if ((size_t)(p-buf) >= sizeof(buf)) {
				yyerror("string too long");
				return findeol();
			}
		} while ((c = lgetc(0)) != EOF && isdigit(c));
		lungetc(c);
		if (p == buf + 1 && buf[0] == '-')
			goto nodigits;
		if (c == EOF || allowed_to_end_number(c)) {
			const char *errstr = NULL;

			*p = '\0';
			yylval.v.number = strtonum(buf, LLONG_MIN,
			    LLONG_MAX, &errstr);
			if (errstr) {
				yyerror("\"%s\" invalid number: %s",
				    buf, errstr);
				return findeol();
			}
			return NUM;
		} else {
nodigits:
			while (p > buf + 1)
				lungetc(*--p);
			c = *--p;
			if (c == '-')
				return c;
		}
	}

#define allowed_in_string(x) \
	(isalnum(x) || (ispunct(x) && x != '(' && x != ')' && \
	x != '{' && x != '}' && \
	x != '!' && x != '=' && x != '#' && \
	x != ',' && x != ';'))

	if (isalnum(c) || c == ':' || c == '_') {
		do {
			*p++ = c;
			if ((size_t)(p-buf) >= sizeof(buf)) {
				yyerror("string too long");
				return findeol();
			}
		} while ((c = lgetc(0)) != EOF && (allowed_in_string(c)));
		lungetc(c);
		*p = '\0';
		if ((token = lookup(buf)) == STRING)
			yylval.v.string = xstrdup(buf);
		return token;
	}
	if (c == '\n') {
		yylval.lineno = file->lineno;
		file->lineno++;
	}
	if (c == EOF)
		return 0;
	return c;
}

struct file *
pushfile(const char *name, int secret)
{
	struct file	*nfile;

	nfile = xcalloc(1, sizeof(*nfile));
	nfile->name = xstrdup(name);
	if ((nfile->stream = fopen(nfile->name, "r")) == NULL) {
		log_warn("can't open %s", nfile->name);
		free(nfile->name);
		free(nfile);
		return NULL;
	}
	nfile->lineno = TAILQ_EMPTY(&files) ? 1 : 0;
	nfile->ungetsize = 16;
	nfile->ungetbuf = xcalloc(1, nfile->ungetsize);
	TAILQ_INSERT_TAIL(&files, nfile, entry);
	return nfile;
}

int
popfile(void)
{
	struct file	*prev;

	if ((prev = TAILQ_PREV(file, files, entry)) != NULL)
		prev->errors += file->errors;

	TAILQ_REMOVE(&files, file, entry);
	fclose(file->stream);
	free(file->name);
	free(file->ungetbuf);
	free(file);
	file = prev;
	return file ? 0 : EOF;
}

int
parse_conf(struct conf *c, const char *filename)
{
	struct sym		*sym, *next;

	default_host = NULL;
	default_port = 1965;

	conf = c;

	file = pushfile(filename, 0);
	if (file == NULL)
		return -1;
	topfile = file;

	yyparse();
	errors = file->errors;
	popfile();

	/* Free macros and check which have not been used. */
	TAILQ_FOREACH_SAFE(sym, &symhead, entry, next) {
		/* TODO: warn if !sym->used */
		if (!sym->persist) {
			free(sym->name);
			free(sym->val);
			TAILQ_REMOVE(&symhead, sym, entry);
			free(sym);
		}
	}

	if (errors)
		return -1;
	return 0;
}

int
symset(const char *name, const char *val, int persist)
{
	struct sym *sym;

	TAILQ_FOREACH(sym, &symhead, entry) {
		if (!strcmp(name, sym->name))
			break;
	}

	if (sym != NULL) {
		if (sym->persist)
			return 0;
		else {
			free(sym->name);
			free(sym->val);
			TAILQ_REMOVE(&symhead, sym, entry);
			free(sym);
		}
	}

	sym = xcalloc(1, sizeof(*sym));
	sym->name = xstrdup(name);
	sym->val = xstrdup(val);
	sym->used = 0;
	sym->persist = persist;

	TAILQ_INSERT_TAIL(&symhead, sym, entry);
	return 0;
}

int
cmdline_symset(char *s)
{
	char	*sym, *val;
	int	 ret;

	if ((val = strrchr(s, '=')) == NULL)
		return -1;
	sym = xcalloc(1, val - s + 1);
	memcpy(sym, s, val - s);
	ret = symset(sym, val + 1, 1);
	free(sym);
	return ret;
}

char *
symget(const char *nam)
{
	struct sym	*sym;

	TAILQ_FOREACH(sym, &symhead, entry) {
		if (strcmp(nam, sym->name) == 0) {
			sym->used = 1;
			return sym->val;
		}
	}
	return NULL;
}

char *
ensure_absolute_path(char *path)
{
	if (path == NULL || *path != '/')
		yyerror("not an absolute path: %s", path);
	return path;
}

int
check_block_code(int n)
{
	if (n < 10 || n >= 70 || (n >= 20 && n <= 29))
		yyerror("invalid block code %d", n);
	return n;
}

char *
check_block_fmt(char *fmt)
{
	char *s;

	for (s = fmt; *s; ++s) {
		if (*s != '%')
			continue;
		switch (*++s) {
		case '%':
		case 'p':
		case 'q':
		case 'P':
		case 'N':
			break;
		default:
			yyerror("invalid format specifier %%%c", *s);
		}
	}

	return fmt;
}

int
check_strip_no(int n)
{
	if (n <= 0)
		yyerror("invalid strip number %d", n);
	return n;
}

int
check_port_num(int n)
{
	if (n <= 0 || n >= UINT16_MAX)
		yyerror("port number is %s: %d",
		    n <= 0 ? "too small" : "too large",
		    n);
	return n;
}

int
check_prefork_num(int n)
{
	if (n <= 0 || n >= PROC_MAX_INSTANCES)
		yyerror("invalid prefork number %d", n);
	return n;
}

void
advance_loc(void)
{
	loc = new_location();
	TAILQ_INSERT_TAIL(&host->locations, loc, locations);
}

void
advance_proxy(void)
{
	proxy = new_proxy();
	TAILQ_INSERT_TAIL(&host->proxies, proxy, proxies);
}

void
parsehp(char *str, char **host, const char **port, const char *def)
{
	char		*at;
	const char	*errstr;

	*host = str;

	if ((at = strchr(str, ':')) != NULL) {
		*at++ = '\0';
		*port = at;
	} else
		*port = def;

	strtonum(*port, 1, UINT16_MAX, &errstr);
	if (errstr != NULL)
		yyerror("port is %s: %s", errstr, *port);
}

int
fastcgi_conf(const char *path, const char *port)
{
	struct fcgi	*f;
	int		i = 0;

	TAILQ_FOREACH(f, &conf->fcgi, fcgi) {
		if (!strcmp(f->path, path) &&
		    ((port == NULL && *f->port == '\0') ||
		     !strcmp(f->port, port)))
			return i;
		++i;
	}

	f = xcalloc(1, sizeof(*f));
	f->id = i;
	(void)strlcpy(f->path, path, sizeof(f->path));
	if (port != NULL)
		(void)strlcpy(f->port, port, sizeof(f->port));
	TAILQ_INSERT_TAIL(&conf->fcgi, f, fcgi);

	return f->id;
}

void
add_param(char *name, char *val)
{
	struct envlist *e;
	struct envhead *h = &loc->params;

	e = xcalloc(1, sizeof(*e));
	(void) strlcpy(e->name, name, sizeof(e->name));
	(void) strlcpy(e->value, val, sizeof(e->value));
	TAILQ_INSERT_TAIL(h, e, envs);
}

int
getservice(const char *n)
{
	struct servent	*s;
	const char	*errstr;
	long long	 llval;

	llval = strtonum(n, 0, UINT16_MAX, &errstr);
	if (errstr) {
		s = getservbyname(n, "tcp");
		if (s == NULL)
			s = getservbyname(n, "udp");
		if (s == NULL)
			return (-1);
		return (ntohs(s->s_port));
	}

	return ((unsigned short)llval);
}

static void
add_to_addr_queue(struct addrhead *a, struct addrinfo *ai)
{
	struct address		*addr;
	struct sockaddr_in	*sin;
	struct sockaddr_in6	*sin6;

	if (ai->ai_addrlen > sizeof(addr->ss))
		fatalx("ai_addrlen larger than a sockaddr_storage");

	TAILQ_FOREACH(addr, a, addrs) {
		if (addr->ai_flags == ai->ai_flags &&
		    addr->ai_family == ai->ai_family &&
		    addr->ai_socktype == ai->ai_socktype &&
		    addr->ai_protocol == ai->ai_protocol &&
		    addr->slen == ai->ai_addrlen &&
		    !memcmp(&addr->ss, ai->ai_addr, addr->slen))
			return;
	}

	addr = xcalloc(1, sizeof(*addr));
	addr->ai_flags = ai->ai_flags;
	addr->ai_family = ai->ai_family;
	addr->ai_socktype = ai->ai_socktype;
	addr->ai_protocol = ai->ai_protocol;
	addr->slen = ai->ai_addrlen;
	memcpy(&addr->ss, ai->ai_addr, ai->ai_addrlen);

	/* for commodity */
	switch (addr->ai_family) {
	case AF_INET:
		sin = (struct sockaddr_in *)&addr->ss;
		addr->port = ntohs(sin->sin_port);
		break;
	case AF_INET6:
		sin6 = (struct sockaddr_in6 *)&addr->ss;
		addr->port = ntohs(sin6->sin6_port);
		break;
	default:
		fatalx("unknown socket family %d", addr->ai_family);
	}

	addr->sock = -1;

	TAILQ_INSERT_HEAD(a, addr, addrs);
}

void
listen_on(const char *hostname, const char *servname)
{
	struct addrinfo hints, *res, *res0;
	int error;

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_flags = AI_PASSIVE;
	error = getaddrinfo(hostname, servname, &hints, &res0);
	if (error) {
		yyerror("listen on \"%s\" port %s: %s", hostname, servname,
		    gai_strerror(errno));
		return;
	}

	for (res = res0; res; res = res->ai_next) {
		add_to_addr_queue(&host->addrs, res);
		add_to_addr_queue(&conf->addrs, res);
	}

	freeaddrinfo(res0);
}
#line 1539 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(void)
{
    unsigned int newsize;
    long sslen;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    sslen = yyssp - yyss;
#ifdef SIZE_MAX
#define YY_SIZE_MAX SIZE_MAX
#else
#define YY_SIZE_MAX 0xffffffffU
#endif
    if (newsize && YY_SIZE_MAX / newsize < sizeof *newss)
        goto bail;
    newss = (short *)realloc(yyss, newsize * sizeof *newss);
    if (newss == NULL)
        goto bail;
    yyss = newss;
    yyssp = newss + sslen;
    if (newsize && YY_SIZE_MAX / newsize < sizeof *newvs)
        goto bail;
    newvs = (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs);
    if (newvs == NULL)
        goto bail;
    yyvs = newvs;
    yyvsp = newvs + sslen;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
bail:
    if (yyss)
            free(yyss);
    if (yyvs)
            free(yyvs);
    yyss = yyssp = NULL;
    yyvs = yyvsp = NULL;
    yystacksize = 0;
    return -1;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse(void)
{
    int yym, yyn, yystate;
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif /* YYDEBUG */

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yyvsp[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 8:
#line 159 "parse.y"
{ file->errors++; }
break;
case 9:
#line 162 "parse.y"
{
			struct file	*nfile;

			if ((nfile = pushfile(yyvsp[0].v.string, 0)) == NULL) {
				yyerror("failed to include file %s", yyvsp[0].v.string);
				free(yyvsp[0].v.string);
				YYERROR;
			}
			free(yyvsp[0].v.string);

			file = nfile;
			lungetc('\n');
		}
break;
case 10:
#line 177 "parse.y"
{ yyval.v.number = 1; }
break;
case 11:
#line 178 "parse.y"
{ yyval.v.number = 0; }
break;
case 12:
#line 181 "parse.y"
{
			if (asprintf(&yyval.v.string, "%s%s", yyvsp[-1].v.string, yyvsp[0].v.string) == -1) {
				free(yyvsp[-1].v.string);
				free(yyvsp[0].v.string);
				yyerror("string: asprintf: %s", strerror(errno));
				YYERROR;
			}
			free(yyvsp[-1].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 14:
#line 194 "parse.y"
{
			char *s;
			if (asprintf(&s, "%d", yyvsp[0].v.number) == -1) {
				yyerror("asprintf: number");
				YYERROR;
			}
			yyval.v.string = s;
		}
break;
case 16:
#line 205 "parse.y"
{
			char *s = yyvsp[-2].v.string;
			while (*s++) {
				if (isspace((unsigned char)*s)) {
					yyerror("macro name cannot contain "
					    "whitespaces");
					free(yyvsp[-2].v.string);
					free(yyvsp[0].v.string);
					YYERROR;
				}
			}
			symset(yyvsp[-2].v.string, yyvsp[0].v.string, 0);
			free(yyvsp[-2].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 17:
#line 222 "parse.y"
{
			if (strlcpy(conf->chroot, yyvsp[0].v.string, sizeof(conf->chroot)) >=
			    sizeof(conf->chroot))
				yyerror("chroot path too long");
			free(yyvsp[0].v.string);
		}
break;
case 18:
#line 228 "parse.y"
{
			yywarn("option `ipv6' is deprecated,"
			    " please use `listen on'");
			if (yyvsp[0].v.number)
				default_host = NULL;
			else
				default_host = "0.0.0.0";
		}
break;
case 20:
#line 237 "parse.y"
{
			yywarn("option `port' is deprecated,"
			    " please use `listen on'");
			default_port = yyvsp[0].v.number;
		}
break;
case 21:
#line 242 "parse.y"
{ conf->prefork = check_prefork_num(yyvsp[0].v.number); }
break;
case 22:
#line 243 "parse.y"
{
			if (tls_config_parse_protocols(&conf->protos, yyvsp[0].v.string) == -1)
				yyerror("invalid protocols string \"%s\"", yyvsp[0].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 23:
#line 248 "parse.y"
{
			if (strlcpy(conf->user, yyvsp[0].v.string, sizeof(conf->user)) >=
			    sizeof(conf->user))
				yyerror("user name too long");
			free(yyvsp[0].v.string);
		}
break;
case 28:
#line 264 "parse.y"
{
			free(conf->log_access);
			conf->log_access = yyvsp[0].v.string;
		}
break;
case 29:
#line 268 "parse.y"
{
			conf->log_format = LOG_FORMAT_COMMON;
		}
break;
case 30:
#line 271 "parse.y"
{
			conf->log_format = LOG_FORMAT_COMBINED;
		}
break;
case 31:
#line 274 "parse.y"
{
			conf->log_format = LOG_FORMAT_CONDENSED;
		}
break;
case 32:
#line 277 "parse.y"
{
			conf->log_format = LOG_FORMAT_LEGACY;
		}
break;
case 33:
#line 280 "parse.y"
{
			const char *str = yyvsp[0].v.string;

			conf->log_syslog = 1;

			if (!strncasecmp(str, "LOG_", 4))
				str += 4;

			if (!strcasecmp(str, "daemon"))
				conf->log_facility = LOG_DAEMON;
#ifdef LOG_FTP
			else if (!strcasecmp(str, "ftp"))
				conf->log_facility = LOG_FTP;
#endif
			else if (!strcasecmp(str, "local1"))
				conf->log_facility = LOG_LOCAL1;
			else if (!strcasecmp(str, "local2"))
				conf->log_facility = LOG_LOCAL2;
			else if (!strcasecmp(str, "local3"))
				conf->log_facility = LOG_LOCAL3;
			else if (!strcasecmp(str, "local4"))
				conf->log_facility = LOG_LOCAL4;
			else if (!strcasecmp(str, "local5"))
				conf->log_facility = LOG_LOCAL5;
			else if (!strcasecmp(str, "local6"))
				conf->log_facility = LOG_LOCAL6;
			else if (!strcasecmp(str, "local7"))
				conf->log_facility = LOG_LOCAL7;
			else if (!strcasecmp(str, "user"))
				conf->log_facility = LOG_USER;
			else
				yywarn("unknown syslog facility `%s'",
				    yyvsp[0].v.string);

			free(yyvsp[0].v.string);
		}
break;
case 34:
#line 316 "parse.y"
{
			conf->log_syslog = 0;
		}
break;
case 35:
#line 319 "parse.y"
{
			conf->log_syslog = 1;
		}
break;
case 36:
#line 324 "parse.y"
{
			host = new_vhost();
			TAILQ_INSERT_HEAD(&conf->hosts, host, vhosts);

			loc = new_location();
			TAILQ_INSERT_HEAD(&host->locations, loc, locations);

			TAILQ_INIT(&host->proxies);

			(void) strlcpy(loc->match, "*", sizeof(loc->match));
			(void) strlcpy(host->domain, yyvsp[0].v.string, sizeof(host->domain));

			if (strstr(yyvsp[0].v.string, "xn--") != NULL) {
				yywarn("\"%s\" looks like punycode: you "
				    "should use the decoded hostname", yyvsp[0].v.string);
			}

			free(yyvsp[0].v.string);
		}
break;
case 37:
#line 342 "parse.y"
{
			if (host->cert_path == NULL ||
			    host->key_path == NULL)
				yyerror("invalid vhost definition: %s",
				    host->domain);
			if (TAILQ_EMPTY(&host->addrs)) {
				char portno[32];
				int r;

				r = snprintf(portno, sizeof(portno), "%d",
				    default_port);
				if (r < 0 || (size_t)r >= sizeof(portno))
					fatal("snprintf");

				yywarn("missing `listen on' in server %s,"
				    " assuming %s port %d", host->domain,
				    default_host ? default_host : "*",
				    default_port);
				listen_on(default_host, portno);
			}
		}
break;
case 38:
#line 363 "parse.y"
{ yyerror("bad server directive"); }
break;
case 43:
#line 372 "parse.y"
{ yyval.v.string = NULL; }
break;
case 45:
#line 376 "parse.y"
{
			struct alist *a;

			a = xcalloc(1, sizeof(*a));
			(void) strlcpy(a->alias, yyvsp[0].v.string, sizeof(a->alias));
			free(yyvsp[0].v.string);
			TAILQ_INSERT_TAIL(&host->aliases, a, aliases);
		}
break;
case 46:
#line 384 "parse.y"
{
			ensure_absolute_path(yyvsp[0].v.string);
			free(host->cert_path);
			host->cert_path = yyvsp[0].v.string;
		}
break;
case 47:
#line 389 "parse.y"
{
			ensure_absolute_path(yyvsp[0].v.string);
			free(host->key_path);
			host->key_path = yyvsp[0].v.string;
		}
break;
case 48:
#line 394 "parse.y"
{
			ensure_absolute_path(yyvsp[0].v.string);
			free(host->ocsp_path);
			host->ocsp_path = yyvsp[0].v.string;
		}
break;
case 49:
#line 399 "parse.y"
{
			yywarn("the top-level `param' directive is deprecated."
			    "  Please use `fastcgi { param ... }`");
			add_param(yyvsp[-2].v.string, yyvsp[0].v.string);
		}
break;
case 50:
#line 404 "parse.y"
{
			listen_on(yyvsp[0].v.string, "1965");
			free(yyvsp[0].v.string);
		}
break;
case 51:
#line 408 "parse.y"
{
			listen_on(yyvsp[-2].v.string, yyvsp[0].v.string);
			free(yyvsp[-2].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 52:
#line 413 "parse.y"
{
			char portno[32];
			int r;

			r = snprintf(portno, sizeof(portno), "%d", yyvsp[0].v.number);
			if (r < 0 || (size_t)r >= sizeof(portno))
				fatal("snprintf");

			listen_on(yyvsp[-2].v.string, portno);
			free(yyvsp[-2].v.string);
		}
break;
case 54:
#line 427 "parse.y"
{ advance_proxy(); }
break;
case 55:
#line 428 "parse.y"
{
			if (*proxy->host == '\0')
				yyerror("invalid proxy block: missing `relay-to' option");

			if ((proxy->cert_path == NULL && proxy->key_path != NULL) ||
			    (proxy->cert_path != NULL && proxy->key_path == NULL))
				yyerror("invalid proxy block: missing cert or key");
		}
break;
case 58:
#line 442 "parse.y"
{ yyval.v.number = 1965; }
break;
case 59:
#line 443 "parse.y"
{
			if ((yyval.v.number = getservice(yyvsp[0].v.string)) == -1)
				yyerror("invalid port number %s", yyvsp[0].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 60:
#line 448 "parse.y"
{ yyval.v.number = yyvsp[0].v.number; }
break;
case 61:
#line 451 "parse.y"
{
			(void) strlcpy(proxy->match_proto, yyvsp[0].v.string, sizeof(proxy->match_proto));
			free(yyvsp[0].v.string);
		}
break;
case 62:
#line 455 "parse.y"
{
			(void) strlcpy(proxy->match_host, yyvsp[-1].v.string, sizeof(proxy->match_host));
			(void) snprintf(proxy->match_port, sizeof(proxy->match_port),
			    "%d", yyvsp[0].v.number);
			free(yyvsp[-1].v.string);
		}
break;
case 65:
#line 467 "parse.y"
{
			free(proxy->cert);
			ensure_absolute_path(yyvsp[0].v.string);
			proxy->cert_path = yyvsp[0].v.string;
		}
break;
case 66:
#line 472 "parse.y"
{
			free(proxy->key);
			ensure_absolute_path(yyvsp[0].v.string);
			proxy->key_path = yyvsp[0].v.string;
		}
break;
case 67:
#line 477 "parse.y"
{
			if (tls_config_parse_protocols(&proxy->protocols, yyvsp[0].v.string) == -1)
				yyerror("invalid protocols string \"%s\"", yyvsp[0].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 68:
#line 482 "parse.y"
{
			(void) strlcpy(proxy->host, yyvsp[-1].v.string, sizeof(proxy->host));
			(void) snprintf(proxy->port, sizeof(proxy->port),
			    "%d", yyvsp[0].v.number);
			free(yyvsp[-1].v.string);
		}
break;
case 69:
#line 488 "parse.y"
{
			ensure_absolute_path(yyvsp[0].v.string);
			proxy->reqca_path = yyvsp[0].v.string;
		}
break;
case 70:
#line 492 "parse.y"
{
			(void) strlcpy(proxy->sni, yyvsp[0].v.string, sizeof(proxy->sni));
			free(yyvsp[0].v.string);
		}
break;
case 71:
#line 496 "parse.y"
{
			proxy->notls = !yyvsp[0].v.number;
		}
break;
case 72:
#line 499 "parse.y"
{
			proxy->noverifyname = !yyvsp[0].v.number;
		}
break;
case 73:
#line 504 "parse.y"
{ advance_loc(); }
break;
case 74:
#line 504 "parse.y"
{
			/* drop the starting '/' if any */
			if (*yyvsp[-4].v.string == '/')
				memmove(yyvsp[-4].v.string, yyvsp[-4].v.string+1, strlen(yyvsp[-4].v.string));
			(void) strlcpy(loc->match, yyvsp[-4].v.string, sizeof(loc->match));
			free(yyvsp[-4].v.string);
		}
break;
case 78:
#line 518 "parse.y"
{ loc->auto_index = yyvsp[0].v.number ? 1 : -1; }
break;
case 79:
#line 519 "parse.y"
{
			check_block_fmt(yyvsp[0].v.string);
			(void) strlcpy(loc->block_fmt, yyvsp[0].v.string, sizeof(loc->block_fmt));
			loc->block_code = check_block_code(yyvsp[-1].v.number);
			free(yyvsp[0].v.string);
		}
break;
case 80:
#line 525 "parse.y"
{
			(void) strlcpy(loc->block_fmt, "temporary failure",
			    sizeof(loc->block_fmt));
			loc->block_code = check_block_code(yyvsp[0].v.number);
			if (yyvsp[0].v.number >= 30 && yyvsp[0].v.number < 40)
				yyerror("missing `meta' for block return %d", yyvsp[0].v.number);
		}
break;
case 81:
#line 532 "parse.y"
{
			(void) strlcpy(loc->block_fmt, "temporary failure",
			    sizeof(loc->block_fmt));
			loc->block_code = 40;
		}
break;
case 82:
#line 537 "parse.y"
{
			(void) strlcpy(loc->default_mime, yyvsp[0].v.string,
			    sizeof(loc->default_mime));
			free(yyvsp[0].v.string);
		}
break;
case 84:
#line 543 "parse.y"
{
			(void) strlcpy(loc->index, yyvsp[0].v.string, sizeof(loc->index));
			free(yyvsp[0].v.string);
		}
break;
case 85:
#line 547 "parse.y"
{
			(void) strlcpy(loc->lang, yyvsp[0].v.string,
			    sizeof(loc->lang));
			free(yyvsp[0].v.string);
		}
break;
case 86:
#line 552 "parse.y"
{ loc->disable_log = !yyvsp[0].v.number; }
break;
case 87:
#line 553 "parse.y"
{
			ensure_absolute_path(yyvsp[0].v.string);
			loc->reqca_path = yyvsp[0].v.string;
		}
break;
case 88:
#line 557 "parse.y"
{
			(void) strlcpy(loc->dir, yyvsp[0].v.string, sizeof(loc->dir));
			free(yyvsp[0].v.string);
		}
break;
case 89:
#line 561 "parse.y"
{ loc->strip = check_strip_no(yyvsp[0].v.number); }
break;
case 92:
#line 566 "parse.y"
{
			loc->fcgi = -1;
			loc->nofcgi = 1;
		}
break;
case 93:
#line 570 "parse.y"
{
			yywarn("`fastcgi path' is deprecated.  "
			    "Please use `fastcgi socket path' instead.");
			loc->fcgi = fastcgi_conf(yyvsp[0].v.string, NULL);
			free(yyvsp[0].v.string);
		}
break;
case 96:
#line 582 "parse.y"
{
			add_param(yyvsp[-2].v.string, yyvsp[0].v.string);
		}
break;
case 97:
#line 585 "parse.y"
{
			loc->fcgi = fastcgi_conf(yyvsp[0].v.string, NULL);
			free(yyvsp[0].v.string);
		}
break;
case 98:
#line 589 "parse.y"
{
			char *c;

			if (asprintf(&c, "%d", yyvsp[0].v.number) == -1)
				fatal("asprintf");
			loc->fcgi = fastcgi_conf(yyvsp[-2].v.string, c);
			free(yyvsp[-2].v.string);
			free(c);
		}
break;
case 99:
#line 598 "parse.y"
{
			loc->fcgi = fastcgi_conf(yyvsp[0].v.string, "9000");
		}
break;
case 100:
#line 601 "parse.y"
{
			loc->fcgi = fastcgi_conf(yyvsp[-2].v.string, yyvsp[0].v.string);
			free(yyvsp[-2].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 101:
#line 606 "parse.y"
{
			loc->fcgi_strip = yyvsp[0].v.number;
		}
break;
case 105:
#line 617 "parse.y"
{
			free(current_media);
			current_media = yyvsp[0].v.string;
		}
break;
case 110:
#line 628 "parse.y"
{
			if (add_mime(&conf->mime, current_media, yyvsp[0].v.string) == -1)
				fatal("add_mime");
			free(yyvsp[0].v.string);
		}
break;
#line 2330 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    if (yyss)
            free(yyss);
    if (yyvs)
            free(yyvs);
    yyss = yyssp = NULL;
    yyvs = yyvsp = NULL;
    yystacksize = 0;
    return (1);
yyaccept:
    if (yyss)
            free(yyss);
    if (yyvs)
            free(yyvs);
    yyss = yyssp = NULL;
    yyvs = yyvsp = NULL;
    yystacksize = 0;
    return (0);
}
